// PSP103 wrappers and models

subckt nmos (d g s b)
parameters w=1u l=0.2u ld=0.5u ls=0.5u
  m (d g s b) psp103n w=w l=l ad=w*ld as=w*ls pd=2*(w+ld) ps=2*(w+ls)
ends

subckt pmos (d g s b)
parameters w=1u l=0.2u ld=0.5u ls=0.5u
  m (d g s b) psp103p w=w l=l ad=w*ld as=w*ls pd=2*(w+ld) ps=2*(w+ls)
ends

model psp103n psp103va (
    type=1
    tr=27.0
    dta=0
    swgeo=1
    qmc=1.0
    lvaro=-10.0e-9
    lvarl=0
    lvarw=0
    lap=10.0e-9
    wvaro=10.0e-9
    wvarl=0
    wvarw=0
    wot=0
    dlq=0
    dwq=0
    vfbo=-1.1
    vfbl=0
    vfbw=0
    vfblw=0
    stvfbo=5.0e-4
    stvfbl=0
    stvfbw=0
    stvfblw=0
    toxo=1.5e-9
    epsroxo=3.9
    nsubo=3.0e+23
    nsubw=0
    wseg=1.5e-10
    npck=1.0e+24
    npckw=0
    wsegp=0.9e-8
    lpck=5.5e-8
    lpckw=0
    fol1=2.0e-2
    fol2=5.0e-6
    facneffaco=0.8
    facneffacl=0
    facneffacw=0
    facneffaclw=0
    gfacnudo=0.1
    gfacnudl=0
    gfacnudlexp=1
    gfacnudw=0
    gfacnudlw=0
    vsbnudo=0
    dvsbnudo=1
    vnsubo=0
    nslpo=0.05
    dnsubo=0
    dphibo=0
    dphibl=0
    dphiblexp=1.0
    dphibw=0
    dphiblw=0
    delvtaco=0
    delvtacl=0
    delvtaclexp=1
    delvtacw=0
    delvtaclw=0
    npo=1.5e+26
    npl=10.0e-18
    cto=5.0e-15
    ctl=4.0e-2
    ctlexp=0.6
    ctw=0
    ctlw=0
    toxovo=1.5e-9
    toxovdo=2.0e-9
    lov=10.0e-9
    lovd=0
    novo=7.5e+25
    novdo=5.0e+25
    cfl=3.0e-4
    cflexp=2.0
    cfw=5.0e-3
    cfbo=0.3
    uo=3.5e-2
    fbet1=-0.3
    fbet1w=0.15
    lp1=1.5e-7
    lp1w=-2.5e-2
    fbet2=50.0
    lp2=8.5e-10
    betw1=5.0e-2
    betw2=-2.0e-2
    wbet=5.0e-10
    stbeto=1.75
    stbetl=-2.0e-2
    stbetw=-2.0e-3
    stbetlw=-3.0e-3
    mueo=0.6
    muew=-1.2e-2
    stmueo=0.5
    themuo=2.75
    stthemuo=-0.1
    cso=1.0e-2
    csl=0
    cslexp=1
    csw=0
    cslw=0
    stcso=-5.0
    xcoro=0.15
    xcorl=2.0e-3
    xcorw=-3.0e-2
    xcorlw=-3.5e-3
    stxcoro=1.25
    fetao=1
    rsw1=50
    rsw2=5.0e-2
    strso=-2.0
    rsbo=0
    rsgo=0
    thesato=1.0e-6
    thesatl=0.6
    thesatlexp=0.75
    thesatw=-1.0e-2
    thesatlw=0
    stthesato=1.5
    stthesatl=-2.5e-2
    stthesatw=-2.0e-2
    stthesatlw=-5.0e-3
    thesatbo=0.15
    thesatgo=0.75
    axo=20
    axl=0.2
    alpl=7.0e-3
    alplexp=0.6
    alpw=5.0e-2
    alp1l1=2.5e-2
    alp1lexp=0.4
    alp1l2=0.1
    alp1w=8.5e-3
    alp2l1=0.5
    alp2lexp=0
    alp2l2=0.5
    alp2w=-0.2
    vpo=0.25
    a1o=1.0
    a1l=0
    a1w=0
    a2o=10.0
    sta2o=-0.5
    a3o=1.0
    a3l=0
    a3w=0
    a4o=0
    a4l=0
    a4w=0
    gcoo=5.0
    iginvlw=50.0
    igovw=10.0
    igovdw=0
    stigo=1.5
    gc2o=1.0
    gc3o=-1.0
    chibo=3.1
    agidlw=50.0
    agidldw=0
    bgidlo=35.0
    bgidldo=41
    stbgidlo=-5.0e-4
    stbgidldo=0
    cgidlo=0.15
    cgidldo=0
    cgbovl=0
    cfrw=5.0e-17
    cfrdw=0
    fnto=1
    nfalw=8.0e+22
    nfblw=3.0e7
    nfclw=0
    rgo=0
    rint=0
    rvpoly=0
    rshg=0
    dlsil=0
    rbulko=0
    rwello=0
    rjundo=0
    rjunso=0
    swjunexp=0
    trj=27.0
    imax=1.0e3
    vjunref=2.5
    fjunq=0.03
    cjorbot=1.0e-3
    cjorsti=1.0e-9
    cjorgat=0.5e-9
    vbirbot=0.75
    vbirsti=1.0
    vbirgat=0.75
    pbot=0.35
    psti=0.35
    pgat=0.6
    phigbot=1.16
    phigsti=1.16
    phiggat=1.16
    idsatrbot=5.0e-9
    idsatrsti=1.0e-18
    idsatrgat=1.0e-18
    csrhbot=5.0e2
    csrhsti=0
    csrhgat=1.0e3
    xjunsti=1.0e-8
    xjungat=1.0e-9
    ctatbot=5.0e2
    ctatsti=0
    ctatgat=1.0e3
    mefftatbot=0.25
    mefftatsti=0.25
    mefftatgat=0.25
    cbbtbot=1.0e-12
    cbbtsti=1.0e-18
    cbbtgat=1.0e-18
    fbbtrbot=1.0e9
    fbbtrsti=1.0e9
    fbbtrgat=1.0e9
    stfbbtbot=-1.0e-3
    stfbbtsti=-1.0e-3
    stfbbtgat=-1.0e-2
    vbrbot=10.0
    vbrsti=10.0
    vbrgat=10.0
    pbrbot=3
    pbrsti=4
    pbrgat=3
    vjunrefd=2.5
    fjunqd=0.03
    cjorbotd=1.0e-3
    cjorstid=1.0e-9
    cjorgatd=1.0e-9
    vbirbotd=1.0
    vbirstid=1.0
    vbirgatd=1.0
    pbotd=0.5
    pstid=0.5
    pgatd=0.5
    phigbotd=1.16
    phigstid=1.16
    phiggatd=1.16
    idsatrbotd=1.0e-12
    idsatrstid=1.0e-18
    idsatrgatd=1.0e-18
    csrhbotd=1.0e+2
    csrhstid=1.0e-4
    csrhgatd=1.0e-4
    xjunstid=1.0e-7
    xjungatd=1.0e-7
    ctatbotd=1.0e+2
    ctatstid=1.0e-4
    ctatgatd=1.0e-4
    mefftatbotd=0.25
    mefftatstid=0.25
    mefftatgatd=0.25
    cbbtbotd=1.0e-12
    cbbtstid=1.0e-18
    cbbtgatd=1.0e-18
    fbbtrbotd=1.0e9
    fbbtrstid=1.0e9
    fbbtrgatd=1.0e9
    stfbbtbotd=-1.0e-3
    stfbbtstid=-1.0e-3
    stfbbtgatd=-1.0e-3
    vbrbotd=10.0
    vbrstid=10.0
    vbrgatd=10.0
    pbrbotd=4
    pbrstid=4
    pbrgatd=4
)

model psp103p psp103va (
    type=-1
    tr=27.0
    dta=0
    swgeo=1
    qmc=1.0
    lvaro=-10.0e-9
    lvarl=0
    lvarw=0
    lap=10.0e-9
    wvaro=10.0e-9
    wvarl=0
    wvarw=0
    wot=0
    dlq=0
    dwq=0
    vfbo=-1.1
    vfbl=0
    vfbw=0
    vfblw=0
    stvfbo=5.0e-4
    stvfbl=0
    stvfbw=0
    stvfblw=0
    toxo=1.5e-9
    epsroxo=3.9
    nsubo=3.0e+23
    nsubw=0
    wseg=1.5e-10
    npck=1.0e+24
    npckw=0
    wsegp=0.9e-8
    lpck=5.5e-8
    lpckw=0
    fol1=2.0e-2
    fol2=5.0e-6
    facneffaco=0.8
    facneffacl=0
    facneffacw=0
    facneffaclw=0
    gfacnudo=0.1
    gfacnudl=0
    gfacnudlexp=1
    gfacnudw=0
    gfacnudlw=0
    vsbnudo=0
    dvsbnudo=1
    vnsubo=0
    nslpo=0.05
    dnsubo=0
    dphibo=0
    dphibl=0
    dphiblexp=1.0
    dphibw=0
    dphiblw=0
    delvtaco=0
    delvtacl=0
    delvtaclexp=1
    delvtacw=0
    delvtaclw=0
    npo=1.5e+26
    npl=10.0e-18
    cto=5.0e-15
    ctl=4.0e-2
    ctlexp=0.6
    ctw=0
    ctlw=0
    toxovo=1.5e-9
    toxovdo=2.0e-9
    lov=10.0e-9
    lovd=0
    novo=7.5e+25
    novdo=5.0e+25
    cfl=3.0e-4
    cflexp=2.0
    cfw=5.0e-3
    cfbo=0.3
    uo=3.5e-2
    fbet1=-0.3
    fbet1w=0.15
    lp1=1.5e-7
    lp1w=-2.5e-2
    fbet2=50.0
    lp2=8.5e-10
    betw1=5.0e-2
    betw2=-2.0e-2
    wbet=5.0e-10
    stbeto=1.75
    stbetl=-2.0e-2
    stbetw=-2.0e-3
    stbetlw=-3.0e-3
    mueo=0.6
    muew=-1.2e-2
    stmueo=0.5
    themuo=2.75
    stthemuo=-0.1
    cso=1.0e-2
    csl=0
    cslexp=1
    csw=0
    cslw=0
    stcso=-5.0
    xcoro=0.15
    xcorl=2.0e-3
    xcorw=-3.0e-2
    xcorlw=-3.5e-3
    stxcoro=1.25
    fetao=1
    rsw1=50
    rsw2=5.0e-2
    strso=-2.0
    rsbo=0
    rsgo=0
    thesato=1.0e-6
    thesatl=0.6
    thesatlexp=0.75
    thesatw=-1.0e-2
    thesatlw=0
    stthesato=1.5
    stthesatl=-2.5e-2
    stthesatw=-2.0e-2
    stthesatlw=-5.0e-3
    thesatbo=0.15
    thesatgo=0.75
    axo=20
    axl=0.2
    alpl=7.0e-3
    alplexp=0.6
    alpw=5.0e-2
    alp1l1=2.5e-2
    alp1lexp=0.4
    alp1l2=0.1
    alp1w=8.5e-3
    alp2l1=0.5
    alp2lexp=0
    alp2l2=0.5
    alp2w=-0.2
    vpo=0.25
    a1o=1.0
    a1l=0
    a1w=0
    a2o=10.0
    sta2o=-0.5
    a3o=1.0
    a3l=0
    a3w=0
    a4o=0
    a4l=0
    a4w=0
    gcoo=5.0
    iginvlw=50.0
    igovw=10.0
    igovdw=0
    stigo=1.5
    gc2o=1.0
    gc3o=-1.0
    chibo=3.1
    agidlw=50.0
    agidldw=0
    bgidlo=35.0
    bgidldo=41
    stbgidlo=-5.0e-4
    stbgidldo=0
    cgidlo=0.15
    cgidldo=0
    cgbovl=0
    cfrw=5.0e-17
    cfrdw=0
    fnto=1
    nfalw=8.0e+22
    nfblw=3.0e7
    nfclw=0
    rgo=0
    rint=0
    rvpoly=0
    rshg=0
    dlsil=0
    rbulko=0
    rwello=0
    rjundo=0
    rjunso=0
    swjunexp=0
    trj=27.0
    imax=1.0e3
    vjunref=2.5
    fjunq=0.03
    cjorbot=1.0e-3
    cjorsti=1.0e-9
    cjorgat=0.5e-9
    vbirbot=0.75
    vbirsti=1.0
    vbirgat=0.75
    pbot=0.35
    psti=0.35
    pgat=0.6
    phigbot=1.16
    phigsti=1.16
    phiggat=1.16
    idsatrbot=5.0e-9
    idsatrsti=1.0e-18
    idsatrgat=1.0e-18
    csrhbot=5.0e2
    csrhsti=0
    csrhgat=1.0e3
    xjunsti=1.0e-8
    xjungat=1.0e-9
    ctatbot=5.0e2
    ctatsti=0
    ctatgat=1.0e3
    mefftatbot=0.25
    mefftatsti=0.25
    mefftatgat=0.25
    cbbtbot=1.0e-12
    cbbtsti=1.0e-18
    cbbtgat=1.0e-18
    fbbtrbot=1.0e9
    fbbtrsti=1.0e9
    fbbtrgat=1.0e9
    stfbbtbot=-1.0e-3
    stfbbtsti=-1.0e-3
    stfbbtgat=-1.0e-2
    vbrbot=10.0
    vbrsti=10.0
    vbrgat=10.0
    pbrbot=3
    pbrsti=4
    pbrgat=3
    vjunrefd=2.5
    fjunqd=0.03
    cjorbotd=1.0e-3
    cjorstid=1.0e-9
    cjorgatd=1.0e-9
    vbirbotd=1.0
    vbirstid=1.0
    vbirgatd=1.0
    pbotd=0.5
    pstid=0.5
    pgatd=0.5
    phigbotd=1.16
    phigstid=1.16
    phiggatd=1.16
    idsatrbotd=1.0e-12
    idsatrstid=1.0e-18
    idsatrgatd=1.0e-18
    csrhbotd=1.0e+2
    csrhstid=1.0e-4
    csrhgatd=1.0e-4
    xjunstid=1.0e-7
    xjungatd=1.0e-7
    ctatbotd=1.0e+2
    ctatstid=1.0e-4
    ctatgatd=1.0e-4
    mefftatbotd=0.25
    mefftatstid=0.25
    mefftatgatd=0.25
    cbbtbotd=1.0e-12
    cbbtstid=1.0e-18
    cbbtgatd=1.0e-18
    fbbtrbotd=1.0e9
    fbbtrstid=1.0e9
    fbbtrgatd=1.0e9
    stfbbtbotd=-1.0e-3
    stfbbtstid=-1.0e-3
    stfbbtgatd=-1.0e-3
    vbrbotd=10.0
    vbrstid=10.0
    vbrgatd=10.0
    pbrbotd=4
    pbrstid=4
    pbrgatd=4
)

